import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
    
/**
 * Class LineDrawer - To draw a given line in a 2D basic graphical panel.
 * 
 * The drawing area is a fixed one, not rescaled for the given line.
 * If the line doesn't cross the drawing zone, an error message is prompted in it.
 * 
 * @author am, modified db, jc, am
 * @version 09/2006
 */

class LineDrawer extends JPanel {

    private final int leftMargin   = 20 ;
    private final int rightMargin  = 20 ;
    private final int topMargin    = 20 ;
    private final int bottomMargin = 20 ;
    private final Color bg = Color.lightGray;
    private final Color fg = Color.black;
    private double a, b, c ; // Coefficients of the line ax+by+c=0 to draw
    
    
    /**
     * Create a 2D graphical panel to draw a given line
     * @param the coefficients a, b, c of a line ax+by+c=0
     */
    public LineDrawer(double _a, double _b, double _c) { 

        // Memorize the coefficients of the line ax+by+c=0 to draw
        a = _a ;  
        b = _b ; 
        c = _c ; 
        
        //Initialize drawing colors, border and opacity
        setBackground(bg);
        setForeground(fg);
        setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createRaisedBevelBorder(),
                BorderFactory.createLoweredBevelBorder()));
    }


    /**
     * Draw the line ax+by+c=0 in a fixed drawing area without scaling.
     * If the line doesn't cross this zone, an error message is prompted.
     */
    public void paintComponent(Graphics g) {
        
        super.paintComponent(g);      //clears the background
     
        // getInsets() = get the external padding of the component (i.e. the
        //   minimum amount of space between the component and the edges of
        //   its display area)
        // getWidth() = get the current width of the component, measured in 
        //   pixels
        Insets insets = getInsets();
        int zeroX = insets.left + leftMargin;
        int zeroY = insets.top + topMargin;
        int maxX  = getWidth() - insets.right - rightMargin ;
        int maxY  = getHeight() - insets.bottom - bottomMargin;
               
        // ******* DRAW AXIS *******
        
        // g.drawLine(x1, y1, x2, y2);
        //   +----> x
        //   |
        //   y
                
        // Draw axis x (horizontal axis)
        g.drawLine(zeroX, zeroY, maxX , zeroY);
        g.drawString("x", maxX+5, zeroY+3);
        // Draw axis y (vertical axis)
        g.drawLine(zeroX, zeroY, zeroX, maxY);
        g.drawString("y", zeroX-10, maxY);  
        // Mark point (0,0)
        g.drawString("(0,0)",zeroX-10, zeroY-3);
        // Draw scale x
        for (int i = 100 ; i < maxX ; i+=100 ) {
            g.drawLine(zeroX+i, zeroY-5, zeroX+i, zeroY+5);
            g.drawString(String.valueOf(i), i+zeroX-7, zeroY-7);
        }
       // Draw scale y
       for (int j = 100 ; j < maxY ; j+=100 ) {
            g.drawLine(zeroX-5, zeroY+j, zeroX+5, zeroY+j);
            g.drawString(String.valueOf(j), zeroX-20, zeroY+j-5);
        }
           
        // ******* CALCULATE THE INTERSECTION POINTS WITH THE BOUNDS OF THE DRAWING AREA *******
        
        double[] Px = new double[2];
        double[] Py = new double[2];
        int nbOfIntersectionPoints = getIntersectionPoints(maxX - zeroX, maxY - zeroY, Px, Py) ;
        
        // ******* DRAW LINE *******
        
        // Does the line cross the drawing area ?
        if ( (nbOfIntersectionPoints==2) && ((Px[0]!=Px[1]) || (Py[0]!=Py[1])) ) {
            // if yes, scale the points and draw the line
            Px[0] = Px[0] + zeroX ;
            Px[1] = Px[1] + zeroX ;
            Py[0] = Py[0] + zeroY ;
            Py[1] = Py[1] + zeroY ;
            g.drawLine((int)Px[0], (int)Py[0], (int)Px[1], (int)Py[1]);
        }
        else { 
            // if no, prompt a message of error in the drawing area
            String equation ;
            equation = a + " x " ;
            if (b >= 0) equation = equation + "+ " + b + " y " ;
            else        equation = equation + b + " y " ;
            if (c >= 0) equation = equation + "+ " + c ;
            else        equation = equation + c ;        
            g.drawString("Line  " + equation + "  is out of drawing bounds",
                         maxX/3, maxY/3) ;
        }

    } // end paintComponent

    
    /**
     * Calculate, if they exist, the intersection points of the line
     * ax+by+c=0 with the bounds of the drawing area.
     * 
     * @param   boundX = max nb of pixels in x
     * @param   boundY = max nb of pixels in y
     * @param   Px = array having to contain the x_coordonates of intersection points
     * @param   Py = array having to contain the y_coordonates of intersection points
     * @return  nb of intersection points ;
     *          Px and Py contain the x and y coordonates of intersection points.
     */
    private int getIntersectionPoints(int boundX, int boundY, double[] Px, double[] Py) {

        int nbPoints = 0 ;
        double x, y ;
      
        if ( b!= 0 ) {
            x = 0 ; 
            y = -c/b ;
            if ( (y>=0) && (y<=boundY) ) {
                Px[nbPoints] = x ;
                Py[nbPoints] = y ;
                nbPoints++;
            }
            x = boundX ;
            y = (-c -a*x)/b;
            if ( (y>=0) && (y<=boundY) ) {
                Px[nbPoints] = x ;
                Py[nbPoints] = y ;
                nbPoints++;
            }
        }

        if ( (a!= 0) && (nbPoints<2) ) {
            y = 0 ; 
            x = -c/a ;
            if ( (x>=0) && (x<=boundX) ) {
                Px[nbPoints] = x ;
                Py[nbPoints] = y ;
                nbPoints++;
            }
            if ( nbPoints<2 ) {
                y = boundY ;
                x = (-c -b*y)/a;
                if ( (x>=0) && (x<=boundX) ) {
                    Px[nbPoints] = x ;
                    Py[nbPoints] = y ;
                    nbPoints++;
                }
            }
        }
        
        return nbPoints;
        
    } // end getIntersectionPoints
    
} // end class LineDrawer